----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

SnowMeltingStationPlaceable = {}
local modDescFile = loadXMLFile("modDesc", g_currentModDirectory.."modDesc.xml");
SnowMeltingStationPlaceable.title = getXMLString(modDescFile, "modDesc.title.en");
SnowMeltingStationPlaceable.author = getXMLString(modDescFile, "modDesc.author");
SnowMeltingStationPlaceable.version = getXMLString(modDescFile, "modDesc.version");
SnowMeltingStationPlaceable.spaceName, _ = Utils.getModNameAndBaseDirectory(g_currentModDirectory.."modDesc.xml");--"FS19_snowMeltingStation";
SnowMeltingStationPlaceable.initialized = false;
SnowMeltingStationPlaceable.drawHelp = false;
delete(modDescFile);

function SnowMeltingStationPlaceable:draw(dt)
	if SnowMeltingStationPlaceable.drawHelp then
		local dirText = "refill"
		local renderHelp = false;
		if g_currentMission.snowMeltingStations ~= nil then
			for _, placeable in pairs(g_currentMission.snowMeltingStations) do
				if placeable.waterRefillVehicle ~= nil and placeable.waterRefillVehicle:getIsActiveForInput(true) then renderHelp = true; end;
				if placeable.refillActivated then dirText = "stop refilling"; break; end;				
			end;
		end;
		if renderHelp then
			local keyString = string.upper(string.gsub(SnowMeltingStationPlaceable.refillKey,"KEY_",""));
			g_currentMission:addExtraPrintText("Press '"..keyString.."' to "..dirText.." Water");
		end;
	end;
end;

function SnowMeltingStationPlaceable:changeWellLevel(plane,dt,raise)
	if raise then
		plane.isHome = false;
		local x,y,z = getTranslation(plane.node);
		y = y+(0.00005*dt);
		if y <= plane.maxHeight then
			setTranslation(plane.node, x,y,z);
		end;
	else			
		local x,y,z = getTranslation(plane.node);
		y = y-(0.000025*dt);
		if y >= plane.home[2] then
			setTranslation(plane.node, x,y,z);
		else						
			setTranslation(plane.node, unpack(plane.home));
			plane.isHome = true;
		end;
	end;
end;

function SnowMeltingStationPlaceable:update(dt)
	if not SnowMeltingStationPlaceable.initialized then
		for _, placeable in pairs(g_currentMission.placeables) do
			if placeable.customEnvironment == SnowMeltingStationPlaceable.spaceName then
			--------------------------------------------------------------------------
				local xmlFile = loadXMLFile("tempXMLFile", placeable.configFileName);
				SnowMeltingStationPlaceable:loadPlaceable(xmlFile, placeable);
				delete(xmlFile);
			---------------------------------------------------------------------------
			end;
			if placeable == g_currentMission.placeables[#g_currentMission.placeables] then
				SnowMeltingStationPlaceable.initialized = true;
			end;
		end;
	end;
	----------------
	if g_currentMission.snowMeltingStations ~= nil then
		for _, placeable in pairs(g_currentMission.snowMeltingStations) do
			----------------
			if placeable.waterRefillVehicle ~= nil then
				local waterTrailer = placeable.waterRefillVehicle;
				if waterTrailer ~= nil then
					if waterTrailer:getIsActiveForInput(true) then
						if Input.isKeyPressed(Input[SnowMeltingStationPlaceable.refillKey]) and SnowMeltingStationPlaceable.refillKeyDelay <= 0 then
							if waterTrailer:getFillUnitFillLevel(waterTrailer.fillUnitIndex) < waterTrailer:getFillUnitCapacity(waterTrailer.fillUnitIndex) then
								placeable.refillActivated = not placeable.refillActivated;
								SnowMeltingStationPlaceable.refillKeyDelay = 18;
							end;
						end;
					end;
					if placeable.refillActivated then
						waterTrailer:addFillUnitFillLevel(g_currentMission.player:getOwnerFarmId(), waterTrailer.fillUnitIndex, waterTrailer.fillLitersPerSecond*dt*0.001, FillType.WATER, ToolType.UNDEFINED, nil);
						--ToolType.TRIGGER
						if not getVisibility(placeable.waterUnloadPipe.effect) then	setVisibility(placeable.waterUnloadPipe.effect, true); end;
						if waterTrailer:getFillUnitFillLevel(waterTrailer.fillUnitIndex) >= waterTrailer:getFillUnitCapacity(waterTrailer.fillUnitIndex) then
							if getVisibility(placeable.waterUnloadPipe.effect) then	setVisibility(placeable.waterUnloadPipe.effect, false); end;
							placeable.refillActivated = false; 
						end;
					end;
				end;
			end;
			----------------
			if placeable.isReceivingSnow > 1 then
				if not placeable.smokeXLPSSystem.isActive then			
					ParticleUtil.setEmittingState(placeable.smokeXLPSSystem, true);
					placeable.smokeXLPSSystem.isActive = true;
				end;
				placeable.isReceivingSnow = placeable.isReceivingSnow - 1.0;				
				SnowMeltingStationPlaceable:changeWellLevel(placeable.waterPlane,dt,true);
				SnowMeltingStationPlaceable:changeWellLevel(placeable.snowPlane,dt,true);
			elseif placeable.isReceivingSnow == 1 then
				if placeable.smokeXLPSSystem.isActive then			
					ParticleUtil.setEmittingState(placeable.smokeXLPSSystem, false);
					placeable.smokeXLPSSystem.isActive = false;
				end;
				placeable.isReceivingSnow = placeable.isReceivingSnow - 1.0;
			else
				if not placeable.snowPlane.isHome then		
					SnowMeltingStationPlaceable:changeWellLevel(placeable.snowPlane,dt,false);
				else					
					if not placeable.waterPlane.isHome then	
						SnowMeltingStationPlaceable:changeWellLevel(placeable.waterPlane,dt,false);
					end;
				end;
			end;
		end;
		----------------
		if SnowMeltingStationPlaceable.refillKeyDelay > 0 then
			SnowMeltingStationPlaceable.refillKeyDelay = SnowMeltingStationPlaceable.refillKeyDelay - 1;
		end;
	end;
	----------------
end;

function SnowMeltingStationPlaceable:waterTriggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	if onEnter then
		for _, placeable in pairs(g_currentMission.snowMeltingStations) do
			if placeable.waterTrigger == triggerId then
				local waterTrailer;
				for _, vehicle in pairs(g_currentMission.vehicles) do
					if vehicle.spec_waterTrailer ~= nil then
						if vehicle.spec_waterTrailer.waterFillNode == otherId then							
							placeable.waterRefillVehicle = vehicle.spec_waterTrailer;	
							SnowMeltingStationPlaceable.drawHelp = true;
							if vehicle.spec_cover ~= nil and vehicle.spec_cover.state ~= 1 then
								vehicle:setCoverState(1, true);
							end;
							break;
						end
					end;
				end;
			end;
		end;
	elseif onLeave then
		for _, placeable in pairs(g_currentMission.snowMeltingStations) do
			if placeable.waterTrigger == triggerId then
				for _, vehicle in pairs(g_currentMission.vehicles) do
					if vehicle.spec_waterTrailer ~= nil then
						if vehicle.spec_waterTrailer.waterFillNode == otherId then
							placeable.waterRefillVehicle = nil;
							placeable.refillActivated = false;
							SnowMeltingStationPlaceable.drawHelp = false;
							if getVisibility(placeable.waterUnloadPipe.effect) then	
								setVisibility(placeable.waterUnloadPipe.effect, false); 
							end;
							break;
						end;
					end;
				end;
			end;
		end;
	end;
end;

function SnowMeltingStationPlaceable:loadPlaceable(xmlFile, placeable)
	local nodeIndex = getXMLString(xmlFile, "placeable.smokeEffect#node");
	placeable.smokePSSystem = {};
	placeable.smokeXLPSSystem = {};
	placeable.exhaustPSSystem = {};
	placeable.smokePSSystem.emitterNode = I3DUtil.indexToObject(placeable.nodeId, nodeIndex);	
	placeable.smokeXLPSSystem.emitterNode = placeable.smokePSSystem.emitterNode;
	local psData = {};
	psData.psFile = "particles/SmokePS.i3d";
	psData.posX, psData.posY, psData.posZ = 0,0,0;
	psData.worldSpace = false;
	ParticleUtil.loadParticleSystemFromData(psData, placeable.smokePSSystem, nil, false, nil, placeable.baseDirectory, placeable.smokePSSystem.emitterNode);
	psData.psFile = "particles/SmokeXLPS.i3d";
	ParticleUtil.loadParticleSystemFromData(psData, placeable.smokeXLPSSystem, nil, false, nil, placeable.baseDirectory, placeable.smokeXLPSSystem.emitterNode);
	placeable.smokeXLPSSystem.isActive = false;			
	placeable.smokePSSystem.isActive = true;	
	ParticleUtil.setEmittingState(placeable.smokeXLPSSystem, false);
	ParticleUtil.setEmittingState(placeable.smokePSSystem, true);
	nodeIndex = getXMLString(xmlFile, "placeable.exhaust#node");
	placeable.exhaustPSSystem.emitterNode = I3DUtil.indexToObject(placeable.nodeId, nodeIndex);
	psData.psFile = "particles/exhaustPS.i3d";
	ParticleUtil.loadParticleSystemFromData(psData, placeable.exhaustPSSystem, nil, false, nil, placeable.baseDirectory, placeable.exhaustPSSystem.emitterNode);
	placeable.exhaustPSSystem.isActive = true;	
	ParticleUtil.setEmittingState(placeable.exhaustPSSystem, true);
	nodeIndex = getXMLString(xmlFile, "placeable.waterTrigger#node");
	placeable.waterTrigger = I3DUtil.indexToObject(placeable.nodeId, nodeIndex);
	placeable.waterUnloadPipe = {}
	nodeIndex = getXMLString(xmlFile, "placeable.refillPipe#node");
	placeable.waterUnloadPipe.node = I3DUtil.indexToObject(placeable.nodeId, nodeIndex);
	nodeIndex = getXMLString(xmlFile, "placeable.refillPipe#effectNode");
	placeable.waterUnloadPipe.effect = I3DUtil.indexToObject(placeable.nodeId, nodeIndex);
	setVisibility(placeable.waterUnloadPipe.effect, false);
	placeable.waterPlane = {};
	nodeIndex = getXMLString(xmlFile, "placeable.waterPlane#node");
	placeable.waterPlane.node = I3DUtil.indexToObject(placeable.nodeId, nodeIndex);
	placeable.waterPlane.home = {getTranslation(placeable.waterPlane.node)};
	placeable.waterPlane.maxHeight = 0.975;
	placeable.waterPlane.isHome = true;
	placeable.snowPlane = {};
	nodeIndex = getXMLString(xmlFile, "placeable.snowPlane#node");
	placeable.snowPlane.node = I3DUtil.indexToObject(placeable.nodeId, nodeIndex);
	placeable.snowPlane.home = {getTranslation(placeable.snowPlane.node)};
	placeable.snowPlane.maxHeight = 1.110;
	placeable.snowPlane.isHome = true;
	placeable.refillActivated = false;
	placeable.isReceivingSnow = 0;
	addTrigger(placeable.waterTrigger, "waterTriggerCallback", SnowMeltingStationPlaceable);
	placeable.sellingStation.unloadTriggers[1].addFillUnitFillLevel = Utils.overwrittenFunction(UnloadTrigger.addFillUnitFillLevel, SnowMeltingStationPlaceable.addFillUnitFillLevel);
	if g_currentMission.snowMeltingStations == nil then g_currentMission.snowMeltingStations = {}; end;
	table.insert(g_currentMission.snowMeltingStations, placeable);
end;

function SnowMeltingStationPlaceable:loadMap(savegame)
	FillTrigger.delete = Utils.appendedFunction(FillTrigger.delete, SnowMeltingStationPlaceable.onDelete);
	SellingStationPlaceable.finalizePlacement = Utils.appendedFunction(SellingStationPlaceable.finalizePlacement, SnowMeltingStationPlaceable.justPurchased);
end;

function SnowMeltingStationPlaceable:justPurchased()
	if SnowMeltingStationPlaceable.initialized then
		--[[for _, placeable in pairs(g_currentMission.placeables) do
			if g_currentMission.snowMeltingStations ~= nil and #g_currentMission.snowMeltingStations > 0 then
				for _, snowMeltingStation in pairs(g_currentMission.snowMeltingStations) do
					if placeable.customEnvironment == SnowMeltingStationPlaceable.spaceName then
						if placeable ~= snowMeltingStation then
						--------------------------------------------------------------------------
							local xmlFile = loadXMLFile("tempXMLFile", placeable.configFileName);
							SnowMeltingStationPlaceable:loadPlaceable(xmlFile, placeable);
							delete(xmlFile);
							return;
						---------------------------------------------------------------------------
						end;
					end;
				end;
			else
				if placeable.customEnvironment == SnowMeltingStationPlaceable.spaceName then
					--------------------------------------------------------------------------
					local xmlFile = loadXMLFile("tempXMLFile", placeable.configFileName);
					SnowMeltingStationPlaceable:loadPlaceable(xmlFile, placeable);
					delete(xmlFile);
					return;
					---------------------------------------------------------------------------
				end;
			end;
		end;]]
		if g_currentMission.snowMeltingStations ~= nil then
			for _, placeable in pairs(g_currentMission.snowMeltingStations) do
				if placeable.waterTrigger ~= nil then
					removeTrigger(placeable.waterTrigger);
				end;
			end;
		end;
		g_currentMission.snowMeltingStations = {};
		SnowMeltingStationPlaceable.initialized = false;
	end;
end;

function SnowMeltingStationPlaceable:onDelete()
	if g_currentMission.snowMeltingStations ~= nil then
		for _, placeable in pairs(g_currentMission.snowMeltingStations) do
			if placeable.waterTrigger ~= nil then
				removeTrigger(placeable.waterTrigger);
			end;
		end;
    end;
end;

function SnowMeltingStationPlaceable:addFillUnitFillLevel(functionRef, farmId, fillUnitIndex, fillLevelDelta, fillTypeIndex, toolType, fillPositionData)
    local applied = self.target:addFillLevelFromTool(farmId, fillLevelDelta, fillTypeIndex, fillPositionData, toolType);
	if applied > 0 then 
		for _, placeable in pairs(g_currentMission.snowMeltingStations) do
			if placeable.sellingStation.unloadTriggers[1] == self then
				placeable.isReceivingSnow = 10;
			end;
		end;
	end;
    return applied;
end;

source(g_currentModDirectory.."getInputKeys.lua");
SnowMeltingStationPlaceable.refillKey = getInputKeys:getMultipleKeys({"ACTIVATE_OBJECT"});
SnowMeltingStationPlaceable.refillKey = getInputKeys:splitKeysFromTable(SnowMeltingStationPlaceable.refillKey[1]);
SnowMeltingStationPlaceable.refillKey = SnowMeltingStationPlaceable.refillKey[1];
SnowMeltingStationPlaceable.refillKeyDelay = 0;
-----------------------------------
print("Load mod: "..SnowMeltingStationPlaceable.title.." : v"..SnowMeltingStationPlaceable.version.." by "..SnowMeltingStationPlaceable.author);
-----------------------------------

addModEventListener(SnowMeltingStationPlaceable);